<?php

namespace App\Http\Controllers;

header("Access-Control-Allow-Origin: *");

use Illuminate\Http\Request;
use Log;
use App\Model\RechargeApi;
use App\Model\IterableApi;
use Carbon\Carbon;
use App\Helper\Helper;

use Session;
use Illuminate\Support\Facades\Auth;

class SubscriptionPillSwapController extends Controller
{
    private $rechargeApi;
    private $iterableObj;
    private $helperObj;
    private $logDirectory;
    private $processedSubscriptions = [];

    public function __construct()
    {
        $this->rechargeApi = new RechargeApi();
        $this->iterableObj = new IterableApi();
        $this->helperObj = new Helper();

        $this->setPaths();
        $this->getLogFilesList();
        $this->setWithLogFiles();
    }

    public function setPaths()
    {
        $this->logDirectory = dirname(__FILE__) . "/../../../public/report/";
    }

    public function getLogFilesList()
    {
        $this->reportFileDirectory = date('dmY');

        $this->logPath = $this->logDirectory . $this->reportFileDirectory;
        if (!is_dir($this->logPath)) {
            //Directory does not exist, so lets create it.
            mkdir($this->logPath);
            chmod($this->logPath, 0777);
        }

        $this->subscriptionSwapReport = "subscription-pill-swap-success" . date("Y_m_d_H_i") . ".csv";

        $this->logFiles = array(
            'subscriptionPillSwap' => array(
                "type"     => "csv",
                "filename" => $this->logPath . "/" . $this->subscriptionSwapReport,
                "mode"     => "",
                "columns"  => array(
                    "email",
                    "subscription_id",
                    "discontinued_pill_sku",
                    "replaced_pill_sku",
                    "previous_subscription_price",
                    "updated_subscription_price",
                    "update_status"
                )
            )
        );
    }

    public function setWithLogFiles()
    {
        if (!empty($this->logFiles)) {

            foreach ($this->logFiles as $logType => $fileDetails) {

                if ($fileDetails['type'] == "csv") {
                    $mode = !empty($fileDetails['mode']) ? $fileDetails['mode'] : "w";

                    if ($mode != "r") {
                        $fileName    = $fileDetails['filename'];
                        $filePointer = fopen($fileName, $mode);
                        $columns     = $fileDetails['columns'];
                        fputcsv($filePointer, $columns);
                        chmod($fileName, 0777);
                        fclose($filePointer);
                    }
                }
            }
        }
    }

    public function getDisContinuePillSubscriptionDetails(Request $request)
    {
        $requestData = $request->input('data');

        if (empty($requestData)) {

            $response = array(
                "status"       => "Failed",
                "message" => "Parameters are missing",
                "error" => "Bad request"
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $requestData = json_decode($requestData);

        $productSku = $requestData->sku;

        if (empty($productSku)) {

            $response = array(
                "status"       => "Failed",
                "message" => "Product SKU is missing",
                "error" => "Bad request"
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }
        $subscriptionData = $this->getAllRechargeSubscriptions($productSku);

        $responseData = [];

        if (count($subscriptionData['subscriptions']) > 0) {
            $responseData['subscriptionCount'] = count($subscriptionData['subscriptions']);
            $responseData['pillCount'] = $subscriptionData['pillCount'];
        } else {
            $responseData['subscriptionCount'] = 0;
            $responseData['pillCount'] = 0;
        }

        $response = array(
            "status"  => "Success",
            "data" => $responseData,
            "message" => $responseData['subscriptionCount'] . ' - Subscriptions found',
            "error"   => ""
        );

        echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
    }


    public function updateSubscriptions(Request $request)
    {
        $requestData = $request->input('data');

        if (empty($requestData)) {

            $response = array(
                "status"       => "Failed",
                "message" => "Parameters are missing",
                "error" => "Bad request"
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $requestData = json_decode($requestData);
        $missingParams = [];

        if (!isset($requestData->discontinuedPill) || empty($requestData->discontinuedPill) || strtolower($requestData->discontinuedPill) == 'none') {

            $missingParams['discontinuedPill'] = "Parameter missing - Discontinued pill";
        }
        if (!isset($requestData->replacePill) || empty($requestData->replacePill || strtolower($requestData->replacePill) == 'none')) {

            $missingParams['replacePill'] = "Parameter missing - replacing pill";
        }

        if (!empty($missingParams)) {
            $response = array(
                "status"       => "Failed",
                "message" => "validation failed",
                "data" => $missingParams,
                "error" => "Bad request"
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $discontinuedPill = trim($requestData->discontinuedPill);
        $replacePill = trim($requestData->replacePill);

        if ($discontinuedPill == $replacePill) {
            $response = array(
                "status"       => "Failed",
                "message" => "same sku",
                "error" => "Bad request"
            );
            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
            die();
        }

        $subscriptionData = $this->getAllRechargeSubscriptions($discontinuedPill);

        if (count($subscriptionData['subscriptions']) > 0 && !empty($subscriptionData['subscriptions'])) {

            $this->swapHistoricalSubscriptionPills($subscriptionData['subscriptions'], $requestData);

            // For sending the report email 
            $this->sendReportForSubscriptionPillSwap();

            $response = array(
                "status"  => "200",
                "message" => "success",
                // "data" => $responseData,
                // "message" => $responseData['subscriptionCount'].' - Subscriptions found',
                "error"   => ""
            );

            echo $_REQUEST['callback'] . '(' . json_encode($response) . ')';
        }
    }

    /**
     * Function for swapping the pills
     * @params Subscription Details and Request data
     */
    public function swapHistoricalSubscriptionPills($subscriptions, $requestData)
    {
        $fileName    = $this->logFiles['subscriptionPillSwap']['filename'];
        $filePointer = fopen($fileName, "a");
        $productDetails = $this->setProductDetails();

        $iterableEventName = !empty(trim($requestData->iterableName)) ? trim($requestData->iterableName) : '';

        $discontinuedSku = trim($requestData->discontinuedPill);
        $replaceSku = trim($requestData->replacePill);

        $discontinuedPillDetails = $productDetails[$discontinuedSku];
        $replacePillDetails = $productDetails[$replaceSku];

        $discontinuedPillName = $discontinuedPillDetails['product_title'];
        $replacedPillName = $replacePillDetails['product_title'];

        $i = 1;
        $testingSubscriptionIds = ['140691199', '60847499', '140699175'];
        foreach ($subscriptions as $subscription) {

            $lineItemSwapValue = '';
            //if (in_array($subscription['subscriptionId'], $testingSubscriptionIds)) {

                $lineItemDetails = $subscription['lineItems'];
                $subscriptionTotalPrice = 0;
                $propertyIndex = 0;
                $subscriptionUpdateRequest = [];
                $pillSwapLineItemName = '';
                $pillSwapLineItemOldValue = '';
                $pillSwapLineItemNewValue = '';
                $subscriptionOldPrice = $subscription['price'];

                // flag for checking the existence of replacing pill
                $replaceSkuExistingStatus = false;

                foreach ($lineItemDetails as $item) {

                    $newSubscriptionPrice = 0;
                    $lineItemQty = 1;
                    $lineItemProductPrice = 0;
                    $explodePropertyName = !empty($item['name']) ? trim(strtolower($item['name'])) : '';
                    if (strpos($explodePropertyName, 'vitamin') !== false) {

                        $explodePropertyValue = explode('|', $item['value']);
                        $lineItemExistingSKU = strtoupper(trim($explodePropertyValue[count($explodePropertyValue) - 1]));

                        // set flag true if replacing already existing in LIP
                        if($lineItemExistingSKU == $replaceSku){
                            $replaceSkuExistingStatus = true;
                        }

                        if ($lineItemExistingSKU == $discontinuedSku) {


                            $pillSwapLineItemName = $explodePropertyName;
                            $pillSwapLineItemOldValue = $item['value'];
                            $newSubscriptionPrice = $replacePillDetails['price'];
                            $explodePropertyValue[count($explodePropertyValue) - 1] = ' ' . trim($replacePillDetails['sku']);
                            // setting new variant Id
                            $explodePropertyValue[count($explodePropertyValue) - 2] = ' ' . trim($replacePillDetails['variant_id']) . ' ';
                            $explodePropertyValue[0] = trim($replacePillDetails['product_title']) . ' ';
                            $explodePropertyValue[1] = ' ' . trim($replacePillDetails['variant_title']) . ' ';

                            $implodePropertyValue = implode('|', $explodePropertyValue);
                            $pillSwapLineItemNewValue = $implodePropertyValue;
                            // echo $i.') Old property value =>'.$item['value'].'<br>';
                            // echo 'new property value =>'.$implodePropertyValue.'<br>';
                            // $i++;
                            $subscription['lineItems'][$propertyIndex]['value'] = $implodePropertyValue;

                            $lineItemQty = !empty($explodePropertyValue[count($explodePropertyValue) - 3]) ?
                                (int)$explodePropertyValue[count($explodePropertyValue) - 3] : 1;
                            $lineItemProductPrice = $newSubscriptionPrice * $lineItemQty;
                            $subscriptionTotalPrice += $lineItemProductPrice;
                        } else {

                            $lineItemSku = $explodePropertyValue[count($explodePropertyValue) - 1];
                            $lineItemSku = trim($lineItemSku);
                            $lineItemProductDetails = $productDetails[$lineItemSku];
                            $newSubscriptionPrice = $lineItemProductDetails['price'];
                            $lineItemQty = !empty($explodePropertyValue[count($explodePropertyValue) - 3]) ?
                                (int)$explodePropertyValue[count($explodePropertyValue) - 3] : 1;
                            $lineItemProductPrice = $newSubscriptionPrice * $lineItemQty;
                            $subscriptionTotalPrice += $lineItemProductPrice;
                        }
                    }
                    $propertyIndex++;
                }

                // If replacing pill existing in LIP re-arranging the same pills in the LIP
                // else no change
                $updatedSubscriptionLineItems = '';
                if($replaceSkuExistingStatus) {
                    $updatedSubscriptionLineItems = $this->checkReplacePillExistence($subscription['lineItems'], $replaceSku );
                } else {
                    $updatedSubscriptionLineItems = $subscription['lineItems'];
                }
                $subscriptionUpdateRequest = array(
                    "price" => $subscriptionTotalPrice,
                    "properties" => $updatedSubscriptionLineItems
                );                
                //dd($subscriptionUpdateRequest);
                $skuUpdatedStatus = $this->rechargeApi->updateSubscription($subscription['subscriptionId'], json_encode($subscriptionUpdateRequest));
                
                if (
                    !empty($skuUpdatedStatus->error) || !empty($skuUpdatedStatus->errors) ||
                    empty($skuUpdatedStatus)
                ) {
                    $error  = "Unknown error";
                    if (!empty($skuUpdatedStatus->error)) {

                        $error = $skuUpdatedStatus->error;
                    } else if (!empty($skuUpdatedStatus->errors)) {

                        $error = "Failed to swap the pill in the subscription";
                    }

                    Log::channel('subscriptionPillSwap')->info(
                        "Failed to swap the pill in the subscription: " .
                            " | Subscription -> " . $subscription['subscriptionId'] .
                            " | Email: " . $subscription['email'] .
                            " | Subscription_lineItem_name: " . $pillSwapLineItemName .
                            " | Subscription_lineItem_previous_value: " . $pillSwapLineItemOldValue .
                            " | Subscription_lineItem_updated_value: " . $pillSwapLineItemNewValue .
                            " | Error -> " . json_encode($error)
                    );
                    $failedSubscription = array(
                        trim($subscription['email']),
                        $subscription['subscriptionId'],
                        $discontinuedSku,
                        $replaceSku,
                        $subscriptionOldPrice,
                        $subscriptionTotalPrice,
                        'Failed to swap the pill - ' . json_encode($error)
                    );
                    fputcsv($filePointer, $failedSubscription);
                    $this->processedSubscriptions[] = $failedSubscription;
                } else {
                    Log::channel('subscriptionPillSwap')->info(
                        "Successfully swapped the pill in the subscription: " .
                            " | Subscription -> " . $subscription['subscriptionId'] .
                            " | Email: " . $subscription['email'] .
                            " | Subscription_lineItem_name: " . $pillSwapLineItemName .
                            " | Subscription_lineItem_previous_value: " . $pillSwapLineItemOldValue .
                            " | Subscription_lineItem_updated_value: " . $pillSwapLineItemNewValue .
                            " | Success "
                    );
                    $successSubscription = array(
                        trim($subscription['email']),
                        $subscription['subscriptionId'],
                        $discontinuedSku,
                        $replaceSku,
                        $subscriptionOldPrice,
                        $subscriptionTotalPrice,
                        'Success'
                    );
                    fputcsv($filePointer, $successSubscription);
                    $this->processedSubscriptions[] = $successSubscription;

                    //To create an iterable event
                    if ($iterableEventName) {
                        $this->iterableEventHandling($subscription['email'], $iterableEventName, $discontinuedPillName, $replacedPillName);
                    }
                }
            //}
        }
    }

    /**
     * To handling the duplicate LIP in the subscription
     */
    public function checkReplacePillExistence($lineItemDetails, $replaceSku) {

        $qty = 0;
        $duplicateItems = [];
        $implodePropertyValue = '';
        $unsetCount = 0;

        foreach ($lineItemDetails as $key => $item) {

            $newSubscriptionPrice = 0;
            $lineItemQty = 1;
            $lineItemProductPrice = 0;
            $explodePropertyName = !empty($item['name']) ? trim(strtolower($item['name'])) : '';
            if (strpos($explodePropertyName, 'vitamin') !== false) {
                $explodePropertyValue = explode('|', $item['value']);
                $lineItemExistingSKU = strtoupper(trim($explodePropertyValue[count($explodePropertyValue) - 1]));

                if ($lineItemExistingSKU == $replaceSku) {

                    $qty += (int)$explodePropertyValue[2];
                    $explodePropertyValue[2] = ' ' . $qty . ' ';
                    $implodePropertyValue = implode('|', $explodePropertyValue);
                    if ($unsetCount >= 1) {
                        unset($lineItemDetails[$key]);
                    }
                    $unsetCount++;
                }
            }
        }
        $j = 1;
        foreach ($lineItemDetails as $vKey => $vItem) {

            $explodePropertyName = !empty($vItem['name']) ? trim(strtolower($vItem['name'])) : '';
            if (strpos($explodePropertyName, 'vitamin') !== false) {

                $explodePropertyValue = explode('|', $vItem['value']);
                $lineItemExistingSKU = strtoupper(trim($explodePropertyValue[count($explodePropertyValue) - 1]));
                if ($lineItemExistingSKU == $replaceSku) {
                    $LIPNewName = 'Vitamin'. ($j<10? '0'.$j++:$j++);
                    $lineItemDetails[$vKey]['name'] = $LIPNewName;
                    $lineItemDetails[$vKey]['value'] = $implodePropertyValue;
                } else {
                    $LIPNewName = 'Vitamin'. ($j<10? '0'.$j++:$j++);
                    $lineItemDetails[$vKey]['name'] = $LIPNewName;
                }
            }
        }
        $lineItemDetails = array_values($lineItemDetails);

        return $lineItemDetails;
    }

    /**
     * Iterable event creation
     */
    public function iterableEventHandling($subscriptionEmail, $iterableEventName, $discontinuedPillName, $replacedPillName)
    {

        $eventRequest = array(
            "events" => array(
                array(
                    "email" => trim($subscriptionEmail),
                    "eventName" => $iterableEventName,
                    "dataFields" => array(
                        "discontinuedPillName" => $discontinuedPillName,
                        "replacedPillName" => $replacedPillName
                    )
                )
            )
        );
        $eventResponse = $this->iterableObj->createNewIterableEvent(json_encode($eventRequest));

        if (isset($eventResponse->successCount) || $eventResponse->successCount > 0) {

            Log::channel('subscriptionPillSwap')->info("Subscription pill swap iterable event - Successfully created event for Email (" .
                $subscriptionEmail . ")");
        } else {
            Log::channel('subscriptionPillSwap')->info("Subscription pill swap iterable event - Failed to create event for Email (" .
                $subscriptionEmail . ")");
        }
    }

    /**
     * Send subscription pill swap report data
     */
    public function sendReportForSubscriptionPillSwap()
    {

        $iterableMailRequest = '';
        $reportSendingTime =  new Carbon(date('Y-m-d H:i:s'), 'America/Los_Angeles');
        $reportSendingTime = $reportSendingTime->toDayDateTimeString();
        $subject = "Nurish-Dev Pill Swap Report - " . $reportSendingTime;

        $recipientEmails = 'testing@p80w.com';
        // $recipientEmails = 'mikegordon79@gmail.com';

        if (count($this->processedSubscriptions) == 0) {
            Log::channel('subscriptionPillSwap')->info("No subscriptions with the selected discontinued pill");
            $commonContent = '<p>No pill swap applied to any of the subscriptions during this execution</p>';

            $iterableMailRequest = array(
                "campaignId" => intval(env('SUBSCRIPTION_PILL_SWAP_APP_CAMPAIGN_ID', true)),
                "recipientEmail" => $recipientEmails,
                "dataFields" => array(
                    "recordDetails" => '',
                    "subject" => $subject,
                    "commonContent" => $commonContent
                ),
            );
        } elseif (count($this->processedSubscriptions) > 30) {

            $fileUrl = url('/') . '/report/' . $this->reportFileDirectory . '/' . $this->subscriptionSwapReport;
            $commonContent = '<p>Please see the list of users who have had their pills 
            swapped according to the pill swap request.</p>';
            $recordDetails = 'Please <a href="' . $fileUrl . '">click here</a> to download the file.';

            $iterableMailRequest = array(
                "campaignId" => intval(env('SUBSCRIPTION_PILL_SWAP_APP_CAMPAIGN_ID', true)),
                "recipientEmail" => $recipientEmails,
                "dataFields" => array(
                    "recordDetails" => $recordDetails,
                    "subject" => $subject,
                    "commonContent" => $commonContent
                ),
            );
        } elseif (count($this->processedSubscriptions) > 0 && count($this->processedSubscriptions) <= 30) {
            $commonContent = '<p>Please see the list of users who have had their pills 
            swapped according to the pill swap request.</p>';

            $recordDetails = '';
            if (!empty($this->processedSubscriptions)) {
                $recordDetails .= "
                <table>
                    <tr>
                        <td>#</td>                
                        <td>Email</td>
                        <td>SubscriptionID</td> 
                        <td>DiscontinuedPillSKU</td> 
                        <td>ReplacedPillSKU</td>
                        <td>PreviousSubscriptionPrice</td>
                        <td>UpdatedSubscriptionPrice</td>   
                        <td>Status</td>                       
                    </tr>            
                ";

                $i = 1;
                foreach ($this->processedSubscriptions as $record) {

                    // $orderScheduledAtDate =  new Carbon($record[3],'America/Los_Angeles');                            
                    // $orderScheduledAtDate = $orderScheduledAtDate->toDayDateTimeString();

                    $recordDetails .= "
                    <tr>
                        <td>" . $i . "</td>                             
                        <td>" . $record[0] . "</td> 
                        <td>" . $record[1] . "</td>  
                        <td>" . $record[2] . "</td>
                        <td>" . $record[3] . "</td>
                        <td>" . $record[4] . "</td>
                        <td>" . $record[5] . "</td>
                        <td>" . $record[6] . "</td>
                    </tr>
                    ";
                    $i++;
                }
                $recordDetails .= '</table>';
            }

            $iterableMailRequest = array(
                "campaignId" => intval(env('SUBSCRIPTION_PILL_SWAP_APP_CAMPAIGN_ID', true)),
                "recipientEmail" => $recipientEmails,
                "dataFields" => array(
                    "recordDetails" => $recordDetails,
                    "subject" => $subject,
                    "commonContent" => $commonContent
                ),
            );
        }

        if (!empty($iterableMailRequest)) {

            $mailTriggerResponse = $this->iterableObj->sendMailFromIterable(json_encode($iterableMailRequest));

            if (strtolower(trim($mailTriggerResponse->code)) == 'success') {

                Log::channel('subscriptionPillSwap')->info("mail send.........................");
            } else {

                Log::channel('subscriptionPillSwap')->info('mail not send......................');
                Log::channel('subscriptionPillSwap')->info(json_encode($mailTriggerResponse));
                $error = $mailTriggerResponse->code;

                if (empty($error)) {

                    $returnData = [
                        'error' => 'failed to send mail(' . $recipientEmails . ')',
                    ];
                } else {

                    $returnData = [
                        'error' => $error . ' (' . $recipientEmails . ')',
                    ];
                }

                Log::channel('subscriptionPillSwap')->info($returnData['error']);
            }
        }
    }

    /**
     * Function to get all the recharge subscriptions
     */
    public function getAllRechargeSubscriptions($productSku = 'A806AB')
    {

        $subscriptionDetails = array();
        $pageNo = 1;
        $limit  = 200;
        $pillCount = 0;
        // $subCount = 1;
        do {
            try {               
                $subscriptions = $this->rechargeApi->getAllSubscriptions($pageNo, $limit);
            } catch (Exception $e) {
                $errorText =
                    "Error form recharge when fetching the subscriptions details and error is -  "
                    . $e->getMessage();
                $this->helperObj->printMe($errorText);
                die;
            }
            if (!empty($subscriptions) && isset($subscriptions)) {

                $subscriptionData = $this->helperObj->objectToArray($subscriptions);

                foreach ($subscriptionData['subscriptions'] as $subscription) {
                    
                    if (!empty($subscription['id'])) {

                        $subCharge = [];
                        $subCharge['email']          = $subscription['email'];
                        $subCharge['subscriptionId'] = $subscription['id'];
                        $scheduledAtDate = new Carbon($subscription['next_charge_scheduled_at']);
                        $scheduledAtDate = $scheduledAtDate->toDateString();
                        $subCharge['scheduledDate']  = $scheduledAtDate;
                        $subCharge['price']  = $subscription['price'];

                        // echo $productSku;die;
                        if (!empty($productSku)) {
                            if (isset($subscription['properties']) && !empty($subscription['properties'])) {

                                $subscriptionItems = $subscription['properties'];
                                $qty = 0;
                                // dd($subscriptionItems);


                                foreach ($subscriptionItems as $value) {
                                    
                                    $propertyName = strtolower(trim($value['name']));

                                    if (strpos($propertyName, 'vitamin') !== false) {

                                        $propertyValue = $value['value'];

                                        if (strpos($propertyValue, '|') !== false) {

                                            $explodePropertyValue = explode('|', $propertyValue);

                                            if (count($explodePropertyValue) >= 1) {

                                                $lineItemExistingSKU = strtoupper(trim($explodePropertyValue[count($explodePropertyValue) - 1]));

                                                if ($productSku == $lineItemExistingSKU) {                                                    

                                                    $subCharge['lineItems']  = $subscriptionItems;                                                    
                                                    
                                                    $qty = (int)$explodePropertyValue[2];
                                                    if ($qty > 1000) {
                                                        $qty = (int)$explodePropertyValue[1];
                                                    }
                                                    $pillCount  += $qty;
                                                    $subscriptionDetails[] = $subCharge;
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }

                        // $subscriptionDetails[] = $subscription;
                    }
                }
                $subscriptionCount = count($subscriptionData['subscriptions']);
            } else {
                break;
            }

            $pageNo++;
        } while ($subscriptionCount == $limit);

        $rechargeSubscriptionDetails['subscriptions'] = $subscriptionDetails;
        $rechargeSubscriptionDetails['pillCount'] = $pillCount;       

        return $rechargeSubscriptionDetails;
    }

    public function setProductDetails()
    {
        $productInStore  = [];
        $limit = 250;
        $nextLink = '';
        $previousLink = '';
        $prodCount = 0;

        $shop    = Auth::user();
        
        do {

            try {

                if (empty($previousLink) && empty($nextLink)) {

                    // $productDetails = $shop->api()->rest(
                    //     'GET',
                    //     '/admin/products.json',
                    //     ["published_status" => "published", "fields" =>
                    //     "id,title,variants", "limit" => $limit]
                    // );
                    $productDetails = $shop->api()->rest(
                        'GET',
                        '/admin/products.json',
                        ["fields" =>"id,title,variants", "limit" => $limit]
                    );


                    $previousLink = !empty($productDetails['link']['previous']) ?
                        $productDetails['link']['previous'] : '';
                    $nextLink = !empty($productDetails['link']['next']) ?
                        $productDetails['link']['next'] : '';
                } elseif (!empty($nextLink)) {

                    $params = [
                        "fields" => "id,title,variants", "limit" => $limit,
                        'page_info' => $nextLink
                    ];
                    $productDetails = $shop->api()->rest('GET', '/admin/products.json', $params);
                    $previousLink = !empty($productDetails['link']['previous']) ?
                        $productDetails['link']['previous'] : '';
                    $nextLink = !empty($productDetails['link']['next']) ?
                        $productDetails['link']['next'] : '';
                }
            } catch (Exception $e) {
                echo PHP_EOL . "Error when fetching the products from shopify and the error is -  "
                    . $e->getMessage();
                die;
            }

            $productDetails = $productDetails['body'];

            if (isset($productDetails['products']) && !empty($productDetails['products'])) {

                $subscription_product = array('4359176454226', '4359186350162', '4426715758674', '4716581486674', '4716582797394');

                foreach ($productDetails['products'] as $product) {
                    if (in_array($product['id'], $subscription_product)) {
                        continue;
                    }


                    foreach ($product['variants'] as $variant) {
                        $prodCount++;

                        if (!empty($variant['sku'])) {
                            $variantDetails = [
                                'variant_id' => $variant['id'],
                                'product_id' => $variant['product_id'],
                                'variant_title' => strip_tags($variant['title']),
                                'product_title' => strip_tags($product['title']),
                                'sku' => $variant['sku'],
                                'price' => $variant['price']
                            ];
                            $productInStore[$variant['sku']] = $variantDetails;
                        }
                    }
                }

                $count = count($productDetails['products']);
            }

            if (empty($nextLink)) {
                break;
            }
        } while ($limit == $count);        

        return $productInStore;
    }
}
